#!/bin/bash

set -e

update_specfile() {
  local specfile
  for specfile in "$@"; do
    local tdir="`mktemp -d`"
    local download_dir="$tdir/dl"
    local source_url="`spectool -s0 \"$specfile\" | sed 's/^Source0: //'`"
    local source_file="$download_dir/${source_url##*/}"
    local source_dir="$tdir/src"
    local go_mod_dir="$tdir/mod"
    local source_list_file="$tdir/srclist"
    local prep_cmds_file="$tdir/prepcmds"
    mkdir "$download_dir"
    curl -s -L -o "$source_file" --url "$source_url"
    mkdir "$source_dir"
    mkdir "$go_mod_dir"
    tar --strip-components=1 -C "$source_dir" -xf "$source_file"
    local rootpath=''
    local -A explicit
    while read k v; do
      [ -z "$rootpath" ] && rootpath="$k"
      [ "$k" = "$rootpath" ] && explicit[$v]=1
    done < <(cd "$source_dir" && GOPATH="$go_mod_dir" go mod graph)
    cp /dev/null "$source_list_file"
    cp /dev/null "$prep_cmds_file"
    local i=100
    local -A sources
    (
      cd "$source_dir"
      GOPATH="$go_mod_dir" go mod download -json |
      jq -r '.Path + " " + .Version'
    ) |
    while read path version; do
      nversion="${version%+incompatible}"
      case "$version" in
      v*-*-*)
        tag="${nversion##*-}"
        ver="$tag"
        ;;
      v*)
        tag="$nversion"
        ver="${nversion#v}"
        ;;
      [0-9]*)
        tag="$nversion"
        ver="$nversion"
        ;;
      *)
        echo "$0: Unknown version format: $version" >&2
        exit 1
        ;;
      esac
      extract_path="$path"
      case "$path" in
      github.com/*)
        repo="${path#github.com/}"
        repo="${repo%%/*}"
        name="${path#github.com/*/}"
        name="${name%%/*}"
        extract_path="github.com/$repo/$name"
        src_url="https://github.com/$repo/$name/archive/$tag/$name-$ver.tar.gz"
        ;;
      golang.org/x/*)
        name="${path##*/}"
        src_url="https://github.com/golang/$name/archive/$tag/$name-$ver.tar.gz"
        ;;
      golang.zx2c4.com/wireguard)
        src_url="https://git.zx2c4.com/wireguard-go/snapshot/wireguard-go-$ver.tar.xz"
        ;;
      golang.zx2c4.com/wireguard/windows)
        src_url="https://git.zx2c4.com/wireguard-windows/snapshot/wireguard-windows-$ver.tar.xz"
        ;;
      esac
      echo "# $path $version" >> "$prep_cmds_file"
      if [ -z "${sources[$src_url]+true}" ]; then
        echo "Source$i: $src_url" >> "$source_list_file"
        sources[$src_url]=$i
        (
          echo "%{__mkdir_p} 'vendor/$extract_path'"
          echo "%{__tar} --strip-components=1 -C 'vendor/$extract_path' -xf %{SOURCE$i}"
        ) >> "$prep_cmds_file"
        ((i++))
      fi
      (
        echo "echo '# $path $version' >> vendor/modules.txt"
        if [ -n "${explicit[$path@$version]+true}" ]; then
          echo "echo '## explicit' >> vendor/modules.txt"
        fi
        echo "find 'vendor/$path' -type f -name '*.go' -print | sed -e 's|^vendor/||' -e 's|/[^/]*\.go$||' | uniq >> vendor/modules.txt"
      ) >> "$prep_cmds_file"
    done
    (
      sed -n -e '1,/^#GO_BUNDLED_SRCS_BEGIN$/p' "$specfile"
      cat "$source_list_file"
      sed -n -e '/^#GO_BUNDLED_SRCS_END$/,/^#GO_BUNDLED_PREP_BEGIN$/p' "$specfile"
      cat "$prep_cmds_file"
      sed -n -e '/^#GO_BUNDLED_PREP_END$/,$p' "$specfile"
    ) > "$specfile.tmp"
    mv "$specfile.tmp" "$specfile"
    find "$go_mod_dir" -type d -print0 | xargs -0 chmod u+rwx
    rm -rf "$tdir"
  done
}

if [ $# -gt 0 ]; then
  update_specfile "$@"
else
  update_specfile *.spec
fi
